<?php

/**
* Tag Groups
*
* @package     Tag Groups
* @author      Christoph Amthor
* @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     GPL-3.0+
* @since      1.8.0
*
*/
if ( !class_exists( 'TagGroups_Group' ) ) {
    class TagGroups_Group
    {
        /**
         * term_group id
         *
         * @var int
         */
        private  $group_id ;
        /**
         * object of groups
         *
         * @var object
         */
        private  $tg_groups ;
        /**
         * array of positions[term_group]
         *
         * @var array
         */
        private  $position ;
        /**
         * array of labels[term_group]
         *
         * @var array
         */
        private  $label ;
        /**
         * instance of the premium part
         *
         * @var object
         */
        private  $tg_group_premium ;
        /**
         * Constructor
         *
         *
         * @param int $group_id optional term_group
         * @return return type
         */
        public function __construct( $group_id = null )
        {
            global  $tag_group_groups, $tag_groups_premium_fs_sdk ;
            $this->tg_groups = $tag_group_groups;
            if ( isset( $group_id ) ) {
                $this->set_group_id( $group_id );
            }
            $this->load();
            return $this;
        }
        
        /**
         * Load data from database
         *
         * @phpunit
         * @param int $group_id optional term_group
         * @return return type
         */
        public function load()
        {
            $labels = $this->tg_groups->get_labels();
            $positions = $this->tg_groups->get_positions();
            
            if ( isset( $labels[$this->group_id] ) ) {
                $this->label = $labels[$this->group_id];
            } else {
                $this->label = '';
            }
            
            
            if ( isset( $positions[$this->group_id] ) ) {
                $this->position = $positions[$this->group_id];
            } else {
                $this->position = 1;
            }
            
            return $this;
        }
        
        /**
         * checks whether this group exists, identified by its ID
         *
         *
         * @phpunit
         * @param void
         * @return boolean
         */
        public function exists()
        {
            if ( 0 == $this->group_id ) {
                return true;
            }
            return isset( $this->group_id ) && in_array( $this->group_id, $this->tg_groups->get_group_ids() );
        }
        
        /**
         * Saves this group to the database
         *
         *
         * @phpunit
         * @param void
         * @return object $this
         */
        public function save()
        {
            if ( empty($this->group_id) ) {
                return $this;
            }
            $labels = $this->tg_groups->get_labels();
            $positions = $this->tg_groups->get_positions();
            $labels[$this->group_id] = $this->label;
            $positions[$this->group_id] = $this->position;
            $this->tg_groups->set_labels( $labels );
            $this->tg_groups->set_positions( $positions );
            $this->tg_groups->save();
            return $this;
        }
        
        /**
         * getter for the term_group value
         *
         *
         * @phpunit
         * @param void
         * @return int term_group
         */
        public function get_group_id()
        {
            return $this->group_id;
        }
        
        /**
         * setter for the term_group value
         *
         *
         * @phpunit
         * @param int $group_id
         * @return object $this
         */
        public function set_group_id( $group_id )
        {
            global  $tag_groups_premium_fs_sdk ;
            $this->group_id = (int) $group_id;
            return $this;
        }
        
        /**
         * adds a new group and saves it
         *
         *
         * @phpunit
         * @param int $position position of the new group; null means after last
         * @param string $label label of the new group
         * @return object
         */
        public function create( $position = null, $label )
        {
            $this->set_group_id( $this->tg_groups->get_max_term_group() + 1 );
            $this->label = $label;
            $this->set_position( $this->tg_groups->get_max_position() + 1 );
            $this->tg_groups->reindex_positions()->add_group( $this );
            if ( isset( $position ) && $position != $this->position ) {
                $this->move_to_position( $position );
            }
            $this->save();
            return $this;
        }
        
        /**
         * returns all terms that are associated with this term group
         *
         * @phpunit
         * @param string|array $taxonomy See get_terms
         * @param string $hide_empty See get_terms
         * @param string $fields See get_terms
         * @param string $post_id See get_terms
         * @param string $orderby See get_terms
         * @param string $order See get_terms
         * @return array|integer
         */
        public function get_group_terms(
            $taxonomy = 'post_tag',
            $hide_empty = false,
            $fields = 'all',
            $post_id = 0,
            $orderby = 'name',
            $order = 'ASC'
        )
        {
            global  $tag_group_terms, $tag_groups_premium_fs_sdk ;
            if ( !isset( $this->group_id ) ) {
                return array();
            }
            $orderby = strtolower( $orderby );
            /**
             * Remove invalid taxonomies
             */
            $taxonomy = TagGroups_Taxonomy::remove_invalid( $taxonomy );
            /*
             * In case we use the Polylang plugin: get the terms for the language of that post.
             * Polylang needs extra query parameter
             */
            
            if ( function_exists( 'pll_get_post_language' ) && $post_id ) {
                /**
                 * Better sanitize what we get from other plugins
                 */
                $pll_post_language = sanitize_text_field( pll_get_post_language( $post_id, 'locale' ) );
            } else {
                $pll_post_language = '';
            }
            
            /**
             * In case we use the WPML plugin: consider the language
             */
            
            if ( defined( 'ICL_LANGUAGE_CODE' ) ) {
                $wpml_language = (string) ICL_LANGUAGE_CODE;
            } else {
                $wpml_language = '';
            }
            
            /**
             * try to get cached version
             *
             * We need to supply the language parameters, because we don't use the Caching class
             */
            $cache_key = md5( $this->group_id . '-' . serialize( $taxonomy ) . '-' . serialize( $hide_empty ) . '-' . strtolower( $fields ) . '-' . $post_id . '-' . strtolower( $orderby ) . '-' . strtolower( $order ) . '-' . $pll_post_language . '-' . $wpml_language );
            $transient_value = TagGroups_Transients::get_transient( TagGroups_WPML::get_tag_groups_group_terms_transient_name() );
            $natural = false;
            
            if ( $transient_value === false || !isset( $transient_value[$cache_key] ) ) {
                
                if ( 'natural' == $orderby ) {
                    $orderby = 'name';
                    $natural = true;
                }
                
                $default_orderby = array(
                    'name',
                    'slug',
                    'term_group',
                    'term_id',
                    'id',
                    'description',
                    'parent',
                    'term_order'
                );
                
                if ( !in_array( $orderby, $default_orderby ) ) {
                    // order by a custom meta field
                    $additional_args = array(
                        'meta_key'     => $orderby,
                        'meta_compare' => 'NUMERIC',
                    );
                    $orderby = 'meta_value_num';
                }
                
                
                if ( 0 == $this->group_id ) {
                    $args = array(
                        'taxonomy'   => $taxonomy,
                        'hide_empty' => $hide_empty,
                        'fields'     => $fields,
                        'orderby'    => $orderby,
                        'order'      => $order,
                        'meta_query' => array(
                        'relation' => 'OR',
                        array(
                        'key'     => '_cm_term_group_array',
                        'value'   => ',0,',
                        'compare' => 'LIKE',
                    ),
                        array(
                        'key'     => '_cm_term_group_array',
                        'compare' => 'NOT EXISTS',
                    ),
                    ),
                    );
                } else {
                    $args = array(
                        'taxonomy'   => $taxonomy,
                        'hide_empty' => $hide_empty,
                        'fields'     => $fields,
                        'orderby'    => $orderby,
                        'order'      => $order,
                        'meta_query' => array( array(
                        'key'     => '_cm_term_group_array',
                        'value'   => ',' . $this->group_id . ',',
                        'compare' => 'LIKE',
                    ) ),
                    );
                }
                
                if ( isset( $additional_args ) ) {
                    $args = array_merge( $args, $additional_args );
                }
                /**
                 * Add Polylang query parameter
                 */
                if ( !empty($pll_post_language) ) {
                    $args['lang'] = $pll_post_language;
                }
                /**
                 * term_order requires special treatment
                 */
                if ( 'term_order' == $orderby ) {
                    add_filter(
                        'get_terms_orderby',
                        array( $tag_group_terms, 'enable_terms_order' ),
                        10,
                        2
                    );
                }
                $terms = get_terms( $args );
                
                if ( !empty($terms) ) {
                    if ( !is_array( $transient_value ) ) {
                        $transient_value = array();
                    }
                    if ( $natural ) {
                        $terms = $tag_group_terms->natural_sorting( $terms, $order );
                    }
                    $transient_value[$cache_key] = $terms;
                    TagGroups_Transients::set_transient( TagGroups_WPML::get_tag_groups_group_terms_transient_name(), $transient_value, 7 * DAY_IN_SECONDS );
                }
                
                return $terms;
            } else {
                return $transient_value[$cache_key];
            }
        
        }
        
        /**
         * adds terms to this group
         *
         *
         * @phpunit
         * @param array $term_ids one-dimensional array of term IDs or WP terms
         * @return object $this
         */
        public function add_terms( $term_ids )
        {
            if ( !is_int( $this->group_id ) ) {
                return $this;
            }
            foreach ( $term_ids as $term_id ) {
                $tg_term = new TagGroups_Term( $term_id );
                $tg_term->add_group( $this->group_id );
                $tg_term->save();
            }
            return $this;
        }
        
        /**
         * removes terms from this group
         *
         *
         * @phpunit
         * @param array $term_ids one-dimensional array of term IDs
         * @return object $this
         */
        public function remove_terms( $term_ids = array() )
        {
            $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();
            if ( empty($term_ids) ) {
                $term_ids = $this->get_group_terms( $enabled_taxonomies, false, 'ids' );
            }
            foreach ( $term_ids as $term_id ) {
                $tg_term = new TagGroups_Term( $term_id );
                $tg_term->remove_group( $this )->save();
            }
            return $this;
        }
        
        /**
         * deletes this group
         *
         *
         * @phpunit
         * @param int $group_id ID of this group
         * @return object $this
         */
        public function delete()
        {
            $labels = $this->tg_groups->get_labels();
            $positions = $this->tg_groups->get_positions();
            $group_ids = $this->tg_groups->get_group_ids();
            if ( ($key = array_search( $this->group_id, $group_ids )) === false ) {
                return $this;
            }
            unset( $group_ids[$key] );
            unset( $labels[$this->group_id] );
            // remove labels of translations
            $tag_group_group_languages = TagGroups_Options::get_option( 'tag_group_group_languages', array() );
            if ( !empty($tag_group_group_languages) ) {
                foreach ( $tag_group_group_languages as $language ) {
                    $translated_labels = TagGroups_Options::get_option( 'term_group_labels_' . $language, array() );
                    
                    if ( is_array( $translated_labels ) ) {
                        unset( $translated_labels[$this->group_id] );
                        update_option( 'term_group_labels_' . $language, $translated_labels );
                    }
                
                }
            }
            unset( $positions[$this->group_id] );
            $this->tg_groups->set_labels( $labels );
            $this->tg_groups->set_positions( $positions );
            $this->tg_groups->set_group_ids( $group_ids );
            $this->tg_groups->reindex_positions();
            $this->tg_groups->save();
            $this->remove_terms();
            do_action( 'term_group_deleted', $this->group_id );
            unset( $this->group_id );
            return $this;
        }
        
        /**
         * returns the position of this group
         *
         * @phpunit
         * @param void
         * @return int|boolean
         */
        public function get_position()
        {
            return $this->position;
        }
        
        /**
         * sets the position of this group
         *
         *
         * @phpunit
         * @param int $position position of this group
         * @return object $this
         */
        public function set_position( $position )
        {
            $this->position = $position;
            return $this;
        }
        
        /**
         * sets the position of this group
         *
         *
         * @phpunit
         * @param int $position position of this group
         * @return object $this
         */
        public function move_to_position( $new_position )
        {
            if ( empty($this->group_id) ) {
                return false;
            }
            $old_position = $this->get_position();
            $positions = $this->tg_groups->get_positions();
            /**
             * 1. move down on old position
             */
            foreach ( $positions as $key => $value ) {
                if ( $value > $old_position ) {
                    $positions[$key] = $value - 1;
                }
            }
            /**
             * 2. make space at new position
             */
            foreach ( $positions as $key => $value ) {
                if ( $value >= $new_position ) {
                    $positions[$key] = $value + 1;
                }
            }
            /**
             * 3. Insert
             */
            $positions[$this->group_id] = $new_position;
            $this->tg_groups->set_positions( $positions );
            $this->position = $new_position;
            $this->tg_groups->reindex_positions();
            return $this;
        }
        
        /**
         * returns the label of this group
         *
         * @phpunit
         * @param void
         * @return string|boolean
         */
        public function get_label()
        {
            if ( !isset( $this->group_id ) ) {
                // allow also "not assigned"
                return false;
            }
            return $this->label;
        }
        
        /**
         * sets the label of this group
         *
         * @phpunit
         * @param string $label label of this group
         * @return object $this
         */
        public function set_label( $label )
        {
            if ( empty($this->group_id) ) {
                return false;
            }
            $this->label = $label;
            return $this;
        }
        
        /**
         * returns the number of terms associated with this group
         *
         * @phpunit
         * @param void
         * @return int
         */
        public function get_number_of_terms( $taxonomies )
        {
            if ( !isset( $this->group_id ) ) {
                return false;
            }
            if ( !is_array( $taxonomies ) ) {
                $taxonomies = array( $taxonomies );
            }
            /**
             * Consider only taxonomies that
             * 1. are among $tag_group_taxonomies
             * 2. actually exist
             */
            $taxonomies = TagGroups_Taxonomy::remove_invalid( $taxonomies );
            return $this->get_group_terms( $taxonomies, false, 'count' );
        }
        
        /**
         * sets $this->group_id by label
         *
         *
         * @phpunit
         * @param string $label
         * @return boolean|int
         */
        public function find_by_label( $label )
        {
            $labels = $this->tg_groups->get_labels();
            
            if ( in_array( $label, $labels ) ) {
                $this->set_group_id( array_search( $label, $labels ) );
                $this->load();
                return $this;
            } else {
                return false;
            }
        
        }
        
        /**
         * sets $this->group_id by position
         *
         *
         * @phpunit
         * @param int $position
         * @return boolean|int
         */
        public function find_by_position( $position )
        {
            $positions = $this->tg_groups->get_positions();
            
            if ( in_array( $position, $positions ) ) {
                $this->set_group_id( array_search( $position, $positions ) );
                $this->load();
                return $this;
            } else {
                $this->set_group_id( 0 );
                return false;
            }
        
        }
        
        /**
         * returns an array of group properties as values
         *
         * @phpunit
         * @param void
         * @return array
         */
        public function get_info(
            $taxonomy = null,
            $hide_empty = false,
            $fields = null,
            $orderby = 'name',
            $order = 'ASC'
        )
        {
            // dealing with NULL values
            if ( empty($fields) ) {
                $fields = 'ids';
            }
            if ( empty($taxonomy) ) {
                $taxonomy = TagGroups_Taxonomy::get_enabled_taxonomies();
            }
            if ( !isset( $hide_empty ) || empty($hide_empty) ) {
                $hide_empty = false;
            }
            $terms = $this->get_group_terms(
                $taxonomy,
                $hide_empty,
                $fields,
                0,
                $orderby,
                $order
            );
            
            if ( !is_array( $terms ) ) {
                $terms = array();
                TagGroups_Error::log( '[Tag Groups] Error retrieving terms in get_info().' );
            }
            
            return array(
                'term_group' => (int) $this->group_id,
                'label'      => $this->label,
                'position'   => (int) $this->position,
                'terms'      => $terms,
            );
        }
        
        /**
         * Returns pieces for for terms_clauses(), enabled for multiple groups
         *
         * @phpunit
         * @param int $group_id
         * @return array
         */
        public function terms_clauses()
        {
            global  $wpdb ;
            
            if ( 0 == $this->group_id ) {
                /**
                 * We are searching for unassigned terms.
                 */
                $meta_query_args = array(
                    'relation' => 'OR',
                    array(
                    'key'     => '_cm_term_group_array',
                    'value'   => ',0,',
                    'compare' => '=',
                ),
                    array(
                    'key'     => '_cm_term_group_array',
                    'compare' => 'NOT EXISTS',
                ),
                );
            } else {
                /**
                 * We are searching for terms assigned to a group with $term_group.
                 */
                $meta_query_args = array(
                    // 'relation' => 'OR',
                    array(
                        'key'     => '_cm_term_group_array',
                        'value'   => ',' . $this->group_id . ',',
                        'compare' => 'LIKE',
                    ),
                );
            }
            
            /**
             * Convert the arguments to SQL pieces
             */
            $meta_query = new WP_Meta_Query( $meta_query_args );
            return $meta_query->get_sql( 'term', 't', 'term_id' );
        }
    
    }
}