<?php

/**
 * @package     Tag Groups
 * @author      Christoph Amthor
 * @copyright   2020 Christoph Amthor (@ Chatty Mango, chattymango.com)
 * @license     GPL-3.0+
 *
 */
if ( !class_exists( 'TagGroups_Hooks' ) ) {
    /**
     *
     */
    class TagGroups_Hooks
    {
        /**
         * does all initialization
         *
         * @var object
         */
        private  $loader ;
        /**
         * enqueues js and css
         *
         * @var object
         */
        private  $enqueue ;
        /**
         * constructor
         *
         * @param object $loader
         */
        function __construct( $loader )
        {
            $this->loader = $loader;
            $this->enqueue = new TagGroups_Enqueue();
        }
        
        /**
         * Runs when plugin is launched
         *
         * @return void
         */
        public function root_all()
        {
            global  $tag_group_terms ;
            /**
             * enable internationalization
             */
            add_action( 'plugins_loaded', array( $this->loader, 'register_textdomain' ) );
            /**
             * Filter that allows to modify the term query and search for terms that have tags in the specified tag groups
             *
             * @param array arguments for WP Term Query
             * @param array|int|string Tag Group IDs (array of integers or comma-separated list of integers)
             * @param string Logical relation between the Tag Group IDs (AND|OR)
             * @return array arguments for WP Term Query
             */
            add_filter(
                'tag_groups_modify_term_query_args',
                array( $tag_group_terms, 'modify_query_args' ),
                10,
                3
            );
        }
        
        /**
         * Runs when plugin is launched and is_admin()
         *
         * @return void
         */
        public function is_admin()
        {
            global  $tag_group_groups, $tag_group_terms ;
            add_action( 'admin_init', array( $this, 'admin_init' ) );
            add_action( 'admin_menu', array( 'TagGroups_Admin', 'register_menus' ) );
            add_action( 'admin_enqueue_scripts', array( $this->enqueue, 'admin_enqueue_scripts' ) );
            /**
             * Display any admin notices from the queue
             */
            add_action( 'admin_notices', array( 'TagGroups_Admin_Notice', 'display' ) );
            /**
             * Add own category to Gutenberg
             */
            add_filter( 'block_categories', array( 'TagGroups_Gutenberg', 'block_categories' ) );
            /**
             * Processing routines in chunks with process bar
             */
            add_action( 'wp_ajax_tg_free_ajax_process', array( 'TagGroups_Process', 'tg_ajax_process' ) );
            /**
             * Processing routines in chunks with process bar
             */
            add_action( 'wp_ajax_tag_groups_get_shortcode_html', array( 'TagGroups_Gutenberg', 'get_shortcode_html' ) );
            // ### Terms ###
            /**
             * After a term has changed its groups, we must update the array of terms per group.
             */
            add_action( 'groups_of_term_saved', array( $tag_group_groups, 'schedule_clear_tag_groups_group_terms' ), 11 );
            /**
             * After a term has been deleted, we might have to update the object cache
             */
            add_action( 'delete_term', array( $tag_group_groups, 'schedule_clear_tag_groups_group_terms' ), 20 );
            /**
             * After a term has been edited, we need to refresh some transients
             */
            add_action( 'edited_term', array( 'TagGroups_Base', 'clear_transients_for_frontend_features' ), 20 );
            /**
             * Plugin: Simple Custom Post Order
             */
            /**
             * After the term order has been changed, we need to refresh some transients
             */
            add_action( 'scp_update_menu_order_tags', array( 'TagGroups_Base', 'clear_transients_for_frontend_features' ), 20 );
            /**
             * Plugin: Custom Taxonomy Order
             */
            /**
             * After the term order has been changed, we need to refresh some transients
             */
            add_action( 'customtaxorder_update_order', array( 'TagGroups_Base', 'clear_transients_for_frontend_features' ), 20 );
            // ### Groups ###
            /**
             *  After a term group order has possibly been changed, we should sort the groups in the term meta
             */
            add_action( 'term_group_saved', array( $tag_group_terms, 'sort_groups' ) );
            /**
             * After a term group has been deleted, we must update the tag meta with the groups.
             */
            add_action( 'term_group_deleted', array( $tag_group_terms, 'remove_missing_groups' ) );
            // ### Taxonomies ###
            /**
             * After the taxonomies have been changed, we check if we must migrate all newly enabled tags.
             */
            add_action( 'taxonomies_saved', array( $tag_group_terms, 'check_if_we_need_to_run_migration' ), 10 );
            /**
             * Rendering a shortcode for the Gutenberg block editor
             */
            add_action( 'wp_ajax_tg_render_shortcode', array( 'TagGroups_Shortcode', 'render' ) );
        }
        
        /**
         * Runs when plugin is launched and ! is_admin()
         *
         * @return void
         */
        public function not_is_admin()
        {
            // frontend stuff
            add_action( 'wp_enqueue_scripts', array( $this->enqueue, 'wp_enqueue_scripts' ) );
            add_action( 'init', array( 'TagGroups_Shortcode', 'maybe_do_shortcode_in_widgets' ) );
        }
        
        /**
         * Runs if is_admin() && on admin_init
         *
         * @return void
         */
        public function admin_init()
        {
            global  $tag_groups_premium_fs_sdk ;
            $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();
            $tag_group_role_edit_tags = 'manage_options';
            $tag_group_role_edit_groups = 'edit_pages';
            if ( current_user_can( $tag_group_role_edit_tags ) ) {
                $this->user_can_tag_group_role_edit_tags( $enabled_taxonomies );
            }
            /**
             * extra columns on tags page
             */
            foreach ( $enabled_taxonomies as $taxonomy ) {
                add_filter( "manage_edit-{$taxonomy}_columns", array( 'TagGroups_Admin', 'add_taxonomy_columns' ) );
                add_filter(
                    "manage_{$taxonomy}_custom_column",
                    array( 'TagGroups_Admin', 'add_taxonomy_column_content' ),
                    10,
                    3
                );
            }
            add_action( 'admin_head', array( 'TagGroups_Admin', 'add_tag_page_styling' ) );
            /**
             * sorting on tags page
             */
            add_filter(
                'terms_clauses',
                array( 'TagGroups_Admin', 'sort_taxonomy_columns' ),
                10,
                3
            );
            /**
             * filtering on tags page
             */
            add_action( 'load-edit-tags.php', array( 'TagGroups_Admin', 'do_filter_tags' ) );
            add_action( 'admin_footer-edit-tags.php', array( 'TagGroups_Admin', 'filter_admin_footer' ) );
            /**
             * Process data submitted from settings forms
             */
            add_action( 'in_admin_header', array( 'TagGroups_Settings', 'settings_page_actions' ) );
            /**
             * Process data submitted from setup wizard forms
             */
            add_action( 'in_admin_header', array( 'TagGroups_Settings', 'settings_page_actions_wizard' ) );
            if ( current_user_can( $tag_group_role_edit_groups ) ) {
                $this->user_can_tag_group_role_edit_groups();
            }
            /**
             * Add link on Plugins page
             */
            add_filter( "plugin_action_links_" . TAG_GROUPS_PLUGIN_BASENAME, array( 'TagGroups_Admin', 'add_plugin_settings_link' ) );
            /**
             * Add post filter
             */
            add_action( 'restrict_manage_posts', array( 'TagGroups_Admin', 'add_post_filter' ) );
            add_filter( 'parse_query', array( 'TagGroups_Admin', 'apply_post_filter' ) );
            /**
             * Register Ajax handler for development feed
             */
            add_action( 'wp_ajax_tg_ajax_get_feed', array( 'TagGroups_Admin', 'ajax_get_feed' ) );
            /**
             * Maybe add message about language-related recommendation
             */
            add_action( 'admin_notices', array( 'TagGroups_Admin', 'add_language_notice' ) );
            /**
             * Add fall-back button to reset the group filter for tags
             */
            add_filter( 'admin_footer_text', array( 'TagGroups_Admin', 'add_admin_footer_text' ), 100 );
        }
        
        /**
         * Functions that require permission tag_group_role_edit_tags
         *
         * @param array $enabled_taxonomies
         * @return void
         */
        private function user_can_tag_group_role_edit_tags( $enabled_taxonomies )
        {
            /**
             * Add group menus when creating and editing tags
             */
            foreach ( $enabled_taxonomies as $taxonomy ) {
                add_action( "{$taxonomy}_edit_form_fields", array( 'TagGroups_Admin', 'render_edit_tag_menu' ) );
                add_action( "{$taxonomy}_add_form_fields", array( 'TagGroups_Admin', 'render_new_tag_menu' ) );
                add_filter(
                    "{$taxonomy}_row_actions",
                    array( 'TagGroups_Admin', 'expand_quick_edit_link' ),
                    10,
                    2
                );
            }
            add_action(
                'quick_edit_custom_box',
                array( 'TagGroups_Admin', 'quick_edit_tag' ),
                10,
                3
            );
            /**
             * Adding tag group menu to quick edit on Tags page
             */
            add_action( 'admin_footer-edit-tags.php', array( 'TagGroups_Admin', 'render_quick_edit_javascript' ) );
            /**
             * A
             */
            add_action( 'admin_footer-edit-tags.php', array( 'TagGroups_Admin', 'bulk_admin_footer' ) );
            /**
             * Processing bulk actions for tags
             */
            add_action( 'load-edit-tags.php', array( 'TagGroups_Admin', 'do_bulk_action' ) );
            /**
             * If using WPML and translation not available, copies tag name to translation 
             */
            add_action( 'create_term', array( 'TagGroups_Admin', 'maybe_copy_term_group_to_translation' ), 20 );
            /**
             * Saves the tag group; other tag info saved anyway
             */
            add_action( 'create_term', array( 'TagGroups_Admin', 'save_term_group_without_tag_info' ) );
            /**
             * Saves the tag group; need to save also other tag info
             */
            add_action( 'edit_term', array( 'TagGroups_Admin', 'save_term_group_with_tag_info' ) );
        }
        
        /**
         * Functions that require permission tag_group_role_edit_groups
         *
         * @return void
         */
        private function user_can_tag_group_role_edit_groups()
        {
            /**
             * Registers Ajax handlers to manage groups on Tag Group Admin page
             */
            add_action( 'wp_ajax_tg_ajax_manage_groups', array( 'TagGroups_Admin', 'ajax_manage_groups' ) );
            add_action( 'wp_ajax_tg_ajax_benchmark', array( 'TagGroups_Settings', 'ajax_benchmark' ) );
        }
    
    }
}